#ifndef LIB_DBG_DEBUG_MQH_INCLUDED
#define LIB_DBG_DEBUG_MQH_INCLUDED
#property version "5.50"
/**********************************************************************************
 * Copyright (C) 2020 Dominik Egert <info@freie-netze.de>
 *
 * This file is the debugger library main include file.
 *
 * Lisence applied: GPLv2
 *                  https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 *
 * Author Dominik Egert / Freie Netze UG.
 **********************************************************************************
 *
 *  Version: 5.01
 *  State: public
 *
 *  File information
 *  ================
 *
 *
 * Include debugging library
 *
 *  Select operational mode:
 *
 *  	- #define LIB_DEBUG
 *  	    -> This will disable LIB_PERF_PROFILING and enable DEBUGGING extensions
 *	
 *  	- #define LIB_MQLAPI_TRACE
 *  	    -> This will enable the MQL API tracer module. Every call to an MQL API fnction will be
 *  	        logged to expert-journal. Also tracing of the _LastError is done.
 *	
 *  	- #define LIB_PERF_PROFILING
 *  	    -> This will enable PERF_PROFILING macros. Only available in "runtime"-mode compilation.
 *  	        If trying to measure these metrics while "_DEBUG" or "LIB_DEBUG" is defined will not work.
 *	
 *  	Define these before you include the library, as they define the included parts of the library.
 *	
 *  	    LIB_DEBUG, LIB_DEBUG_AUTOENABLE, LIB_DEBUG_MQLAPI_TRACE and
 *  	        LIB_MQLAPI_TRACE are superior to LIB_PERF_PROFILING and will disable LIB_PERF_PROFILING.
 *
 *
 *  Additional options for LIB_DEBUG
 *
 *		LIB_DEBUG_AUTOENABLE
 *
 *			This flag will make the library switch between runtime and debugging mode, depending on 
 *			the compiler target. - This will imply LIB_DEBUG automatically as appropiate.
 *
 *		LIB_DEBUG_MQLAPI_TRACE
 *
 *  		MQL API function call tracing support. This will log calls to built-in functions.
 *  		"LIB_DEBUG_MQLAPI_TRACE" will enable this feature. You will get trace output, if you have
 *  		LIB_DEBUG built-in tracing for the particular function enabled. - See example below on how to 
 *  		make a function traceable.
 *  		
 *  		    Functions in which the tracing is activated, see description below, will now
 *  		    also log calls to MQL-API functions. The error code and return type will be logged.
 *  		
 *  		If you wish this feature to be enabled regardless of function tracing, you may define the flag
 *  		"LIB_MQLAPI_TRACE". Now all calls to MQL-API get logged and checked. If this is
 *  		activated, "LIB_DEBUG_MQLAPI" is implicated automatically, if LIB_DEBUG or LIB_DEBUG_AUTOENABLE
 *  		is active.
 *	
 *		LIB_DEBUG_LOGFILE
 *
 *  		Debugging library supports output logging to file. This feature can be enabled by defining
 *  		"LIB_DEBUG_LOGFILE" as shown below. (Automatically also defines LIB_DEBUG)
 *  		    Logs will be written to Common/* directory.
 *		
 *  		    This feature supports debug logging within optimizer runs of the program.
 *	
 *		LIB_DEBUG_NO_JOURNAL_OUTPUT
 *
 *  		Additionally the expert-journal output can be surpressed, separating the logs into two
 *  		targets. - This way logging debug output and logging terminal output is separated.
 *		
 *  		Define "LIB_DEBUG_NO_JOURNAL_OUTPUT" to disable expert-journal output from this library.
 *
 *
 *  Additional options for LIB_DEBUG_MQLAPI_TRACE and LIB_MQLAPI_TRACE
 *
 *		To always print input parameters to a function, define
 *      LIB_MQLAPI_TRACE_SHOW_PARAMS. 
 *
 *          This will force printing of all input parameters 
 *          which are not output parameters only.
 *
 *      LIB_MQLAPI_TRACE_SHOW_RESULT
 *
 *          This will force printing of results from the function call, including
 *          current error state
 *
 *
 *  !!! IMPORTANT NOTE !!!
 *
 *      MQLAPI_TRACE and LIB_DEBUG_MQLAPI_TRACE is incompatible with the standard library from MQL.
 *      Work to resolve this issue is underway, but might take quite a while.
 *
 *      Until then, if you are using any includes from standard library, you cannot activate 
 *      the MQLAPI tracer module. - You will get compiler errors.
 *
 *      Same goes for any function you created on your own, that shares a name with any of 
 *      the functions provided by MQL-API. 
 *
 *      The tracer uses macro substitution to replace and intercept the calls to the API.
 *      Therefore any function with the same name as listed in the file
 *          "MQLplus/lib_debug/lib_debug_mqlapi_tracer_overwrite_macros.mqh"
 *      will confuse the compiler and library. You will not be able to compile your code.
 *
 *      If it is mandatory for you to do so anyways, you need to setup a custom config
 *      for your project. - See below.
 *
 *  !!! END !!!
 *
 *	
 *	Customization for project specific needs
 *
 *		If you wish to customize the library, this can be done easily. Procedure is described in
 *			"MQLplus/lib_debug/lib_debug_CustomConfig.mqh"
 *	
 *		See details in that file to customize lib_debug to your needs.
 *
 *
 *
 *
 *  Detailed usage:
 *
 *      Initial setup of library to enable its functionality
 *
 *      It is required to set these macro definitions before including the library file.
 *
 *          [... main .mq5-file ...]
 *
 *          #define LIB_DEBUG                                               // This enables the librarys fuinctionality
 *          #define LIB_DEBUG_LOGFILE                                       // This enables the librarys fuinctionality and will copy all output to a logfile.
 *                                                                          // The logfile is located in the common-folder and follows naming: common/debug_log/[PROGRAM_NAME]/{timestamp of starting the program}.txt
 *                                                                          // Timestamp is bound to the local computer clock, unix timestamp.
 *          #define LIB_DEBUG_NO_JOURNAL_OUTPUT                             // Surpress printing to the terminals journal
 *          #include <../Shared Projects/MQLplus/lib_debug.mqh>             // Depending on where you place the file, this include path will take it directly from the shared storage.
 *
 *          [... your code follows here ...]
 *
 *
 *      Defines macros for supporting debugging code.
 *
 *      Macros beginning with DBG_STR_* will represent a string.
 *      Macros beginning with DBG_MSG_* will printf information.
 *
 *      Following output and string macros are defined:
 *
 *          DBG_STR(x)                                                      // Trace info prefix.
 *          DBG_STR_PERSIST(x)                                              // Trace info prefix, message string will persist into release build.
 *          DBG_STR_VAR(x)                                                  // Convert variable value to string
 *          DBG_STR_VAR_IF(cond, var)                                       // Convert variable on condition (condition, variable)
 *          DBG_STR_PERSIST(x)                                              // Message string will persist into release build.
 *          DBG_STR_BITS(x)                                                 // Convert a variable and its binary representation to a string
 *          DBG_STR_ERRCODE(x)                                              // Convert an error code to the error constant
 *
 *          DBG_MSG(var)                                                    // Will print out a text prefixed with trace information
 *          DBG_MSG_IF(cond)                                                // Print text on condition (condition)
 *          DBG_MSG_VAR(var)                                                // Print a variable
 *          DBG_MSG_VAR_IF(cond, var)                                       // Print a variable on condition (condition, variable)
 *          DBG_MSG_PERSIST(x)                                              // Print message string will persist into release build.
 *          DBG_MSG_BITS(x)                                                 // Print a variable and its binary representation
 *          DBG_MSG_ERRCODE(x)                                              // Print the error constant identified by the given error code
 *
 *          DBG_MSG_EVAL(bool_expr)                                         // Print boolean evaluation within an if, for, while or ()?: statement
 *          DBG_MSG_EVAL_IF(condition, bool_expr)                           // Print boolean evaluation within an if, for, while or ()?: statement, if condition is true
 *          DBG_MSG_EVAL_CMNT(cmnt, bool_expr)                              // Print boolean evaluation within an if, for, while or ()?: statement, add a value as comment
 *          DBG_MSG_EVAL_IF_CMNT(condition, cmnt, bool_expr)                // Print boolean evaluation within an if, for, while or ()?: statement, add a value as comment, if condition is met
 *
 *          DBG_MSG_TRACE_BEGIN                                             // Output >Trace begin<
 *          DBG_MSG_TRACE_END                                               // Output >Trace end<
 *          DBG_MSG_TRACE_RETURN                                            // Replace return(void);
 *          DBG_MSG_TRACE_RETURN_VAR(x)                                     // Replace return(x);
 *
 *          DBG_MSG_TRACE_FILE_LOADER                                       // (Only MQL5) Traces the loading order of source code files, place the macro at the top of a file.
 *                                                                          // As the file gets loaded, it will print out the filename in the experts journal.
 *
 *
 *      Vardump for arrays:
 *
 *          DBG_MSG_LISTDUMP(array1, array2)                                // Dump two arrays side by side 
 *
 *
 *      Check index of array:
 *
 *          DBG_MSG_ARRAY_OUT_OF_RANGE(array, index)                        // Dump two arrays side by side 
 *
 *
 *      To support assert debugging code, following macros are defined:
 *
 *          DBG_ASSERT(condition, message)                                  // Default assert macro. Will end execution by crash code.
 *          DBG_ASSERT_LOG(condition, message)                              // Logging only assert macro. Execution will continue normally.
 *          DBG_ASSERT_RETURN_VOID(condition, message)                      // Return assert macro. Will execute a function return if condition is met.
 *          DBG_ASSERT_RETURN(condition, message, return_value)             // Return assert macro. Will execute a function return if condition is met and return a given value/variable.
 *
 *
 *      Software Break points:
 *
 *          DBG_SLEEP_SECONDS(seconds)                                      // Will insert a realtime sleep into execution
 *          DBG_SOFT_BREAKPOINT                                             // Will halt execution for given timeout
 *          DBG_SOFT_BREAKPOINT_TS(timestamp)                               // Will halt for timeout seconds at (TimeCurrent() - (TimeCurrent() % PeriodSeconds()) == timestamp)
 *          DBG_SOFT_BREAKPOINT_CONDITION(x)                                // Will halt execution if condition is met
 *          DBG_SOFT_BREAKPOINT_EXEC_TIME(x)                                // Will halt execution after given execution time runtime
 *
 *
 *      Condtional Break points and groups:
 *
 *          Initially you create a break ID on the global scope of the program, using the macro
 *          DBG_BREAK_CONDITION_CREATE. The first parameter is any alphanumeric ID you make up
 *          to identify the break point group. The second parameter allows you to disable the group
 *          and therefore should be set to true by default.
 *          Within the code, you use the other macros. The macro DBG_BREAK_CONDITION_ACTIVATE is used to
 *          activate a break point group. if the y parameter is true, it will activate the brakepoints
 *          with the identifier given in the parameter x.
 *          The macro DBG_BREAK_CONDITION_DEACTIVATE allows you to disable a breakpoint group by code. If
 *          its parameter y is true, it will disable the group.
 *          The macro DBG_BREAK_CONDITION_ON_ID is used to have breakpoints spread out in your code,
 *          wherever this macro is placed and when the group condition has been evaluated to true, the break
 *          point will be enabled and halt execution on this macro.
 *
 *          The parameter x is the group id, set by you, alphanumeric. No quotations!!
 *          The parameter y is the conditional parameter and must evaluate to true or false. It can be
 *          a function call, a variable or an evaluation. Following the boolean evaluation of the compiler.
 *
 *          DBG_BREAK_CONDITION_CREATE(x, y)                                // Create a conditional break ID group
 *          DBG_BREAK_CONDITION_ACTIVATE(x, y)                              // Enable the groups break points
 *          DBG_BREAK_CONDITION_DEACTIVATE(x, y)                            // Disable a groups break points
 *          DBG_BREAK_CONDITION_ON_ID(x)                                    // Check for break on this group
 *
 *
 *      Call counter break point:
 *
 *          Function calling is being counted by the macro DBG_MSG_TRACE_BEGIN and provides a counter variable
 *          specific to this function. A break point for debugging can be set with the following macro, enabling you
 *          to interrupt the program at a specific call counter. Specify the count as parameter to this macro.
 *
 *          DBG_BREAK_CALL_COUNTER(x)                                       // Break point after x calls to this function
 *
 *
 *      Loop tracing:
 *
 *          DBG_TRACE_LOOP_BEGIN                                            // A new loop will be defined
 *          DBG_TRACE_LOOP_START                                            // At the beginning of a loop, inside the loop
 *          DBG_TRACE_LOOP_FINISH                                           // At the end of a loop, inside the loop
 *          DBG_TRACE_LOOP_END                                              // After a loop is done, will print out stats
 *
 *
 *      Complex loop tracing:
 *
 *          To have multiple execution paths withina loop being seperately traced, you may
 *          want to specify the trace id. This way you can define different paths throughout the loop
 *          and get an understanding of the execution costs each path implies.
 *
 *          DBG_TRACE_LOOP_BEGIN_ID(id)                                     // A new loop counter with id will be defined
 *          DBG_TRACE_LOOP_START_ID(id)                                     // Start loop counter with ID at the beginning of a loop, inside the loop
 *          DBG_TRACE_LOOP_FINISH_ID(id)                                    // Finish loop counter with ID at the end of a loop, inside the loop
 *          DBG_TRACE_LOOP_END_ID(id)                                       // After a loop is done, will print out stats for this ID
 *
 *
 *  Hint:
 *
 *      All debug macros will be removed in runtime builds.
 *
 *
 *  Example:
 *
 *      Function definition for call tracing:
 *
 *          Define two macros for replacement to be able to turn on/off tracing
 *          for a specific function.
 *          To enable tracing a define will switch trace code on.
 *
 *          #define DBG_TRACE_SOME_FUNCTION
 *
 *          Later in the function body, the macro xxx_RETURN will be used instead
 *          of the usual "return" command...
 *
 *          /////////////////////////////////////
 *          // Function debug trace code
 *
 *          #ifdef DBG_TRACE_SOME_FUNCTION
 *              #undef DBG_TRACE_SOME_FUNCTION
 *              #define DBG_TRACE_SOME_FUNCTION(x) x
 *              #define DBG_TRACE_SOME_FUNCTION_RETURN(x) DBG_MSG_TRACE_RETURN_VAR(x)
 *          #else
 *              #define DBG_TRACE_SOME_FUNCTION(x)
 *              #define DBG_TRACE_SOME_FUNCTION_RETURN(x) DBG_MSG_NOTRACE_RETURN_VAR(x)
 *          #endif
 *
 *          /////////////////////////////////////
 *
 *          Return value example:
 *
 *              #define xxx_RETURN(x) DBG_MSG_TRACE_RETURN_VAR(x)
 *
 *              Here the return value will be for example an integer.
 *
 *              When using Performance Counters, the Return Macro gets extended
 *              with the macro PERF_COUNTER_END in RELEASE. This will print out
 *              the statistics of the functions performance counter value.
 *
 *
 *          double some_function(const int index = NULL)
 *          {
 *              DBG_TRACE_SOME_FUNCTION(
 *                  DBG_MSG_TRACE_BEGIN;
 *                  DBG_MSG_VAR(index)
 *                  );
 *              PERF_COUNTER_BEGIN;
 *
 *              DBG_TRACE_SOME_FUNCTION(DBG_TRACE_LOOP_BEGIN);
 *              for(int cnt = (int)some_periods; ((cnt > NULL) && (!_StopFlag)); cnt--)
 *              {
 *                  DBG_TRACE_SOME_FUNCTION(DBG_TRACE_LOOP_START);
 *
 *                  // Basic calculations
 *                  a += a * cnt;
 *
 *                  // Some check
 *                  if(a)
 *                  { continue; }
 *
 *                  DBG_TRACE_SOME_FUNCTION(DBG_TRACE_LOOP_FINISH);
 *              }
 *              DBG_TRACE_SOME_FUNCTION(DBG_TRACE_LOOP_END);
 *
 *              // Assert value is greater than NULL
 *              DBG_ASSERT((some_testing_value > NULL), DBG_MSG_VAR(some_testing_value));
 *
 *              // Return
 *              DBG_TRACE_SOME_FUNCTION_RETURN(some_double_value);
 *          }
 *
 *
 *      Notice:
 *
 *          This line of code is the function return command.
 *          When passing a NULL value to a macro, you need to
 *          cast the NULL-Macro according to the return value type
 *          like this:
 *
 *              // Return
 *              DBG_TRACE_SOME_FUNCTION_RETURN((double)NULL);
 *
 *          Else you will receive a complie error.
 *
 *
 *      VOID-Function signatures
 *
 *          Return type "void" uses a different type of macro, replace DBG_MSG_TRACE_RETURN_VAR
 *          with DBG_MSG_TRACE_RETURN
 *
 *          /////////////////////////////////////
 *          // Function debug trace code
 *
 *          #ifdef DBG_TRACE_SOME_FUNCTION
 *              #undef DBG_TRACE_SOME_FUNCTION
 *              #define DBG_TRACE_SOME_FUNCTION(x) x
 *              #define DBG_TRACE_SOME_FUNCTION_RETURN DBG_MSG_TRACE_RETURN
 *          #else
 *              #define DBG_TRACE_SOME_FUNCTION(x)
 *              #define DBG_TRACE_SOME_FUNCTION_RETURN DBG_MSG_NOTRACE_RETURN
 *          #endif
 *
 *          /////////////////////////////////////
 *
 *
 *  Performance tracing
 *
 *      These macros are used to have performance indications on
 *      the execution speed of a function or block of code.
 *
 *      The macros without parameter are specificated on the function
 *      itself, if more than one counter within the same function or
 *      execution block is needed, they can be created by using the
 *      macros with the parameter. Pass any type of ID to it and use
 *      it later to close this counter again.
 *
 *      Performance macros will only be available in RELEASE versions.
 *      They are stripped of in debugging as well from release, if disabled.
 *
 *      Defined performance macros
 *
 *          PERF_COUNTER_BEGIN                  // Begin measurement of execution time
 *          PERF_COUNTER_END                    // End performance measurement and show stats
 *
 *          PERF_COUNTER_DEFINE_ID(x)           // Create function global performance counter
 *          PERF_COUNTER_BEGIN_ID(x)            // Begin measurement of execution time
 *          PERF_COUNTER_END_ID(x)              // End performance measurement and show stats
 *
 *          PERF_COUNTER_TIMEIT_V(x)            // Macro for measuring functions with "void" return values
 *          PERF_COUNTER_TIMEIT_R(x)            // Macro for measuring functions having valid return values
 *          PERF_COUNTER_TIMEIT_O(x)            // Macro for measuring functions returning objects as result.
 *                                              //  NOTE: Copy-Constructor of the returned object will be called twice!
 *                                                        First call will be part of measurement.
 *
 *
 *  EX5 Program info
 *
 *      A special macro is defined to deliver program information and can be
 *      displyed ie at program start. (Very helpful to determin if the program is
 *      running in debug or release mode)
 *
 *              DBG_STR_EX45_FILEINFO           // EX4/EX5 program information
 *
 *
 *  Available configuration options:
 *
 *      Basic settings are exposed below. Here is a full list of options:
 *
 *          Functional options:
 *
 *              LIB_DEBUG                       // Force enable debugging support
 *              LIB_DEBUG_LOGFILE               // Enable logging all output to file
 *              LIB_DEBUG_NO_JOURNAL_OUTPUT     // Quiet mode, will surpress all messages to the terminals expert-journal
 *              LIB_PERF_PROFILING              // Enable performance counters (Only in release mode)
 *              DBG_SOFT_BKP_TIMEOUT            // Breakpoint timeout before continuing execution, for soft break-points (a type of sleep-command)
 *              DBG_TRACE_EXEC_DELAY_MS         // When tracing function calls, this will insert a delay in execution after each function with enabled tracing.
 *              DBG_CRASH_CODE                  // Code used to crash the program if an assert() fails
 *
 *          Output format options:
 *
 *              DBG_ASSERT_MSG_TXT              // Prefix to assert messages
 *              DBG_TRACE_BEGIN_MSG_TXT         // Trace function call begin prefix
 *              DBG_TRACE_END_MSG_TXT           // Trace function call end prefix
 *              DBG_OUTPUT_PREFIX               // General debug output prefix
 *              DBG_OUTPUT_FORMAT               // Format string applied to debug output
 *              DBG_DOUBLE_OUTPUT_DIGITS        // Digits debugger will output on double values
 *
 *      If not defined, the default applies automatically.
 *
 *          Following list of defaults is configured as preset:
 *
 *              DBG_SOFT_BKP_TIMEOUT            120                                                     // Wait 2 minutes
 *              DBG_TRACE_EXEC_DELAY_MS         0x00                                                    // Disabled by default
 *              DBG_CRASH_CODE                  { int arr[1]; int i = 1; i = (arr[1] / (i - i)); }      // Zero divide abnormal program termination
 *
 *              DBG_ASSERT_MSG_TXT              "### --- Assertion failed!"
 *              DBG_TRACE_BEGIN_MSG_TXT         ">>>Function begin<<<"
 *              DBG_TRACE_END_MSG_TXT           ">>>Function end<<<"
 *              DBG_OUTPUT_PREFIX               "   "                                                   // Four white spaces
 *              DBG_OUTPUT_STRING               "%s%s, %s, line: %i%s"
 *              DBG_OUTPUT_FORMAT               DBG_OUTPUT_STRING, ((prefix == NULL) ? NULL : prefix + ", "), __FILE__, __FUNCSIG__, __LINE__, ((message == NULL) ? NULL : ", " + message)
 *              DBG_FLOAT_OUTPUT_DIGITS         _Digits
 *              DBG_DOUBLE_OUTPUT_DIGITS        _Digits
 *
*/
//#property strict


//////////////////////////////////////////
//
// Debugger switches and definitions
//

// Force enable debugger
//#define LIB_DEBUG

// Enable logging to file
// #define LIB_DEBUG_LOGFILE

// Disable logging to terminals journal
//#define LIB_DEBUG_NO_JOURNAL_OUTPUT

// Enable performance tracer
//#define LIB_PERF_PROFILING

// Enable MQL API function call tracing
//#define LIB_MQLAPI_CALL_TRACING_ENABLED

// Force disable MQL API function call tracing
//#define LIB_MQLAPI_CALL_TRACING_DISABLE

// Pause execution after a vardump call
//#define DBG_VARDUMP_PAUSE 0

// Soft breakpoint timeout in seconds
//#define DBG_SOFT_BKP_TIMEOUT 120

// Execution delay inserted into trace runs in milliseconds
//#define DBG_TRACE_EXEC_DELAY_MS 25

// Crash code definition
//#define DBG_CRASH_CODE { int arr[1]; int i = 1; i = (arr[1] / (i - i)); }

// Array element listing limits
//#define DBG_ARRAY_LIST_MAX 25
//#define DBG_ARRAY_LIST_HEADTAIL_COUNT 5

//
//////////////////////////////////////////







//*********************************************************************************************************************************************************/
// Debugging support
//



/////////////////////////////////////////////////////////////////////////////////////////////////////
//
// General library helper macros 
//

#define DBG_PREPROCESSOR_CONCAT_STRING(x, y)                    x##y

//
/////////////////////////////////////////////////////////////////////////////////////////////////////





/////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Auto enable/disable LIB_DEBUG by environment
//

// Autoenable LIB_DEBUG
#ifdef _DEBUG
    #ifdef LIB_DEBUG_AUTOENABLE
        #ifndef LIB_DEBUG
            #define LIB_DEBUG

        #endif
        #ifndef DBG_DEBUGGER_FLAG
            #define DBG_DEBUGGER_FLAG ((LIB_DBG_NAMESPACE(dbg_lib, dbg_MQLInfoInteger)(MQL_DEBUG)) ? "MQL_DEBUG set (autoenable)" : "_DEBUG defined (autoenable)")
    
        #endif

    #endif
    #ifndef DBG_DEBUGGER_FLAG
        #define DBG_DEBUGGER_FLAG ((LIB_DBG_NAMESPACE(dbg_lib, dbg_MQLInfoInteger)(MQL_DEBUG)) ? "MQL_DEBUG set" : "_DEBUG defined")

    #endif

#endif

// Disable all DEBUG_* features in runtime
#ifdef LIB_DEBUG_AUTOENABLE
    #ifndef LIB_DEBUG
        #undef LIB_DEBUG_LOGFILE
        #undef LIB_DEBUG_NO_JOURNAL_OUTPUT
        #undef LIB_DEBUG_MQLAPI_TRACE
        #undef LIB_MQLAPI_TRACE
        
    #endif

#endif

// Check for api-tracer
#ifdef LIB_DEBUG_MQLAPI_TRACE
    #ifndef LIB_DEBUG
        #define LIB_DEBUG
        #define LIB_DEBUG_INCLUDE_STUB

    #endif

#endif

// Enable debugging support
#ifdef LIB_DEBUG_LOGFILE
    #ifndef LIB_DEBUG
        #define LIB_DEBUG

    #endif
    #ifndef LIB_DBG_LOG_TO_FILE
        #define LIB_DBG_LOG_TO_FILE

    #endif

#endif

// Disable no logging, if file logging is not enabled
#ifdef LIB_DEBUG_NO_JOURNAL_OUTPUT
    #ifndef LIB_DBG_LOG_TO_FILE
        #undef LIB_DEBUG_NO_JOURNAL_OUTPUT
        
    #endif

#endif

// Update _DEBUG flag
#ifndef _DEBUG
    #ifdef LIB_DEBUG
        #define _DEBUG true
        #ifdef _RELEASE
            #undef _RELEASE
            
        #endif

    #endif

#endif

//
/////////////////////////////////////////////////////////////////////////////////////////////////////





/////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Selective enable/disable LIB_MQLAPI_TRACER
//

// Force enable by MQLAPI
#ifdef LIB_MQLAPI_TRACE
    #ifndef LIB_DEBUG_MQLAPI_ALL_CALLS
        #define LIB_DEBUG_MQLAPI_ALL_CALLS

    #endif

#endif
#ifdef LIB_DEBUG_MQLAPI_ALL_CALLS
    #ifndef LIB_DEBUG_MQLAPI
        #define LIB_DEBUG_MQLAPI

    #endif

#endif
#ifdef LIB_DEBUG_MQLAPI_TRACE
    #ifndef LIB_DEBUG_MQLAPI
        #define LIB_DEBUG_MQLAPI

    #endif

#endif
#ifdef LIB_DEBUG_MQLAPI
    #ifndef LIB_DEBUG
        #define LIB_DEBUG
        #define LIB_DEBUG_INCLUDE_STUB

    #endif

#endif

// By default disable mql api tracing
#ifndef LIB_DEBUG_MQLAPI
    #ifndef LIB_MQLAPI_CALL_TRACING_DISABLE
        #define LIB_MQLAPI_CALL_TRACING_DISABLE

    #endif

#else
    #ifndef LIB_MQLAPI_CALL_TRACING_ENABLED
        #define LIB_MQLAPI_CALL_TRACING_ENABLED

    #endif

#endif

//
/////////////////////////////////////////////////////////////////////////////////////////////////////





/////////////////////////////////////////////////////////////////////////////////////////////////////
//
// MQL4 MQL5 environment adoption
//

// Environment check
#ifdef __MQL4__
    #ifndef __MQL4_COMPATIBILITY_CODE__
        #ifndef __NO_MQL4_COMPATIBILITY_CODE__
            #define __NO_MQL4_COMPATIBILITY_CODE__

        #endif

    #endif
    #ifndef LIB_DBG_NAMESPACE
        #define LIB_DBG_NAMESPACE(x, y)         x##_##y

    #endif
    #ifndef LIB_DBG_NAMESPACE_DEF
        #define LIB_DBG_NAMESPACE_DEF(x, y)     LIB_DBG_NAMESPACE(x, y)

    #endif
#else
    #ifndef LIB_DBG_NAMESPACE
        #define LIB_DBG_NAMESPACE(x, y)         x::y

    #endif
    #ifndef LIB_DBG_NAMESPACE_DEF
        #define LIB_DBG_NAMESPACE_DEF(x, y)     y

    #endif
#endif

// Custom MQLplus expert remove reason
#ifndef REASON_MQLPLUS_EXPERT_KILL
    #define REASON_MQLPLUS_EXPERT_KILL          0xFF01

#endif

//
/////////////////////////////////////////////////////////////////////////////////////////////////////





/////////////////////////////////////////////////////////////////////////////////////////////////////
//
// LIB_DEBUG / LIB_MQLAPI_TRACE
//

#ifdef LIB_DEBUG
    /////////////////////////////////////////////////////////////////////////////////////////////////////
    //
    // Debugger mode
    //
    #ifdef LIB_DEBUG_INCLUDE_STUB
        #include <MQLplus/lib_debug/lib_debug_disable.mqh>
        #include <MQLplus/lib_debug/lib_debug_definitions.mqh>

    #else
        #include <MQLplus/lib_debug/lib_debug_definitions.mqh>
        #include <MQLplus/lib_debug/lib_debug_macros.mqh>

    #endif
    #include <MQLplus/lib_debug/lib_debug_function_trace_macros.mqh>
    #include <MQLplus/lib_debug/lib_debug_global_program_tracer.mqh>
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLERS
        #include <MQLplus/lib_debug/lib_debug_global_event_handler_tracer.mqh>

    #endif
    #include <MQLplus/lib_debug/lib_debug_functions.mqh>
    #include <MQLplus/lib_debug/lib_debug_type_to_string.mqh>
    #include <MQLplus/lib_debug/lib_debug_printf.mqh>


    //
    /////////////////////////////////////////////////////////////////////////////////////////////////////





    /////////////////////////////////////////////////////////////////////////////////////////////////////
    //
    // MT4/5 API function call tracing
    //
    #ifdef LIB_DEBUG_MQLAPI
        #ifndef __MQL4__
            #include <MQLplus/lib_debug/lib_debug_mqlapi_tracer_mql5_stdlib_injector.mqh>

        #endif
        #include <MQLplus/lib_debug/lib_debug_mqlapi_definitions.mqh>
        #include <MQLplus/lib_debug/lib_debug_mqlapi_tracer.mqh>
        #include <MQLplus/lib_debug/lib_debug_mqlapi_tracer_mql45_vararg.mqh>
        #include <MQLplus/lib_debug/lib_debug_mqlapi_tracer_mql45.mqh>

        #ifndef __MQL4__
            #include <MQLplus/lib_debug/lib_debug_mqlapi_tracer_mql5.mqh>

        #else
            #include <MQLplus/lib_debug/lib_debug_mqlapi_tracer_mql4.mqh>

        #endif


        ////////////////////////////////////////
        //
        // Create MQL-API tracer object
        //
        static LIB_DBG_MQLAPI_TRACER_OBJECT             dbg_mql_api_call_tracer();


        ////////////////////////////////////////
        //
        // Include STDLIB tracer object
        //
        #ifdef LIB_MQLAPI_TRACE_STDLIB_EXTENSION
            #ifndef __MQL4__
                #include <MQLplus/lib_debug/lib_debug_mqlapi_tracer_mql5_stdlib.mqh>

            #endif
        
        #endif


        ////////////////////////////////////////
        //
        // Include MQL API overwrite macros
        //
        #include <MQLplus/lib_debug/lib_debug_mqlapi_tracer_overwrite_macros.mqh>

    #endif
    //
    /////////////////////////////////////////////////////////////////////////////////////////////////////





    /////////////////////////////////////////////////////////////////////////////////////////////////////
    //
    // Disable performance counters for debugging
    //
    #include <MQLplus/lib_debug/lib_perf_counter_disable.mqh>


    //
    /////////////////////////////////////////////////////////////////////////////////////////////////////





/////////////////////////////////////////////////////////////////////////////////////////////////////
//
// LIB_PERF_COUNTERS
//

#else
    /////////////////////////////////////////////////////////////////////////////////////////////////////
    //
    // Runtime code setup
    //
    #include <MQLplus/lib_debug/lib_debug_disable.mqh>


    //
    /////////////////////////////////////////////////////////////////////////////////////////////////////





    /////////////////////////////////////////////////////////////////////////////////////////////////////
    //
    // Performance profiling support
    //
    #ifdef LIB_PERF_PROFILING
        #include <MQLplus/lib_debug/lib_perf_counter_macros.mqh>
        #include <MQLplus/lib_debug/lib_perf_counter_functions.mqh>

    #else
        #include <MQLplus/lib_debug/lib_perf_counter_disable.mqh>

    #endif
    //
    /////////////////////////////////////////////////////////////////////////////////////////////////////

#endif


//
// END Debugging support
//*********************************************************************************************************************************************************/
#endif // LIB_DEBUG_MQH_INCLUDED
